/*
**  SLOPE_FOR: C-XSC classes for one- and multidimensional (interval)
**             slope arithmetic in forward mode
**
**  Copyright (C) 1999-2001 M. Braeuer, W. Hofschuster, W. Kraemer
**                          Wiss. Rechnen/Softwaretechnologie
**                          Universitaet Wuppertal, Germany   
**
**  This library is free software; you can redistribute it and/or
**  modify it under the terms of the GNU Library General Public
**  License as published by the Free Software Foundation; either
**  version 2 of the License, or (at your option) any later version.
**
**  This library is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
**  Library General Public License for more details.
**
**  You should have received a copy of the GNU Library General Public
**  License along with this library; if not, write to the Free
**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "interval.hpp"
#include "imatrix.hpp"
#include "rmatrix.hpp"

#include <iostream>

#include "slope_ari.h" //mehrdim. Steigungsarithmetik einbinden

using namespace cxsc;
using namespace std; 

//-------------------------Defektbetrachtung--------------------------------
//
//Parameter : b=60, alpha =1/10, m={2,6,8,10} 
//
//--------------------------------------------------------------------------

interval alpha; //=0.1

void init_alpha()
{
  // char* x="0.1";
 
  // real x_l, x_u;
  //"0.1" >> cin >> SaveOpt >> RndDown >> x_l;
  //"0.1" >> cin >> RndUp >> x_u >> RestoreOpt;

  //alpha = _interval(x_l, x_u);
  alpha = interval(1) / interval(10);
}

/*---------------------------------------------------------------------------

Funktion w definieren

---------------------------------------------------------------------------*/
slope w(slope x)
{
 interval pi=_real(4.0)*atan(_interval(1.0)); //pi bestimmen

 int m=2;
 interval b=_interval(60);
 
 slope res;

 res = alpha*sin(b*pi*x); //i=1
 for(int i=2; i<=m; i++) res=res+power(alpha,i)*sin(power(b,i)*pi*x);

 real c_,_c;
 m=m+1;
 c_=Sup( power(alpha,m)/(_interval(1.0)-alpha) ); // c etwas groesser gemacht
 _c=-c_;
 interval c=interval(_c,c_);

 res=res+c; //Fehlerintervall aufaddieren

 return res;
}

/*---------------------------------------------------------------------------

Funktion f=(f1,f2) definieren, fuer die Nullstellen verifiziert werden sollen

---------------------------------------------------------------------------*/
slope f1(slope_vector x)
{
 slope res;

 res=exp(w(x[1]))-w(x[2])-_interval(1.0);

 return res;
}

slope f2(slope_vector x)
{
 slope res;
 res=sqr(w(x[1]))+sqr(w(x[2]))
     -cos(x[1]*_interval(20))*cos(x[2]*_interval(20))/sqrt(_interval(2.0));
 return res;
}

//-------------------------------------------------------------------------

void verif_schritt(const ivector& x_ein)
{
   slope_vector x;
   slope_vector y(1,2,2);

   ivector  x_approx(2);

   x_approx[1]=mid(x_ein[1]);
   x_approx[2]=mid(x_ein[2]);

   ivector y_ein(2);
  
   y_ein[1]=x_ein[1]-x_approx[1]; //y=X-c Defekt
   y_ein[2]=x_ein[2]-x_approx[2];    

   x=InitVar(x_ein, x_approx); //zur Berechnung von sf(c, y+c)=sf(c, X)
                               //init.: X=x_ein, c=x_approx

   //print_slope(x[1]);
   //print_slope(x[2]); 

   y[1]=f1(x); //Steigungsmatrix von f berechnen
   y[2]=f2(x);

   ivector s1(2); s1=get_fs(y[1]);
   ivector s2(2); s2=get_fs(y[2]);  
   
   //cout << s1 << " " << s2 << " " << endl;

   /*------------------------------------------------------------------------
   I, M, R berechnen / initialisieren
   -------------------------------------------------------------------------*/ 

   rmatrix I(2,2), M(2,2), R(2,2); 

   real det;

   imatrix slopemx(2,2);

   I[1][1]=1.0 ;   I[1][2]=0.0;
   I[2][1]=0.0 ;   I[2][2]=1.0;

   M[1][1]=mid(s1[1]);   M[1][2]=mid(s1[2]);
   M[2][1]=mid(s2[1]);   M[2][2]=mid(s2[2]);   

   //nun: R:=inv(M)

   det=M[1][1]*M[2][2]-M[1][2]*M[2][1];

   if(det==0) cout << "Matrix nicht invertierbar " << endl<< M << endl;

   R[1][1]=M[2][2]/det;    R[1][2]=-M[1][2]/det;
   R[2][1]=-M[2][1]/det;   R[2][2]=M[1][1]/det;
   
   slopemx[1][1]=s1[1]; slopemx[1][2]=s1[2];
   slopemx[2][1]=s2[1]; slopemx[2][2]=s2[2];

   /*------------------------------------------------------------------------
   Funktionswert bestimmen
   -------------------------------------------------------------------------*/ 
   ivector fx_approx(2);
  
   fx_approx[1]= get_fx(f1(Init_mid_Var(x_approx)));       
   fx_approx[2]= get_fx(f2(Init_mid_Var(x_approx)));

   /*------------------------------------------------------------------------
   Nun Fixpunkt verifizieren
   -------------------------------------------------------------------------*/
   ivector ynew(2);

   ynew=-R*fx_approx+(I-R*slopemx)*y_ein;

   /*------------------------------------------------------------------------
   Ausgabe
   -------------------------------------------------------------------------*/

   cout << "Argumentintervalle: " << endl;
   cout << "Xein[1]: " << x_ein[1] << endl;
   cout << "Xein[2]: " << x_ein[2] << endl << endl; 

   if (Inf(ynew[1]) > Inf(y_ein[1])) 
     cout << " erste Komp. untere Schranke:  Ja" << endl;
   else
     cout << " erste Komp. untere Schranke:  Nein" << endl;

  if (Sup(ynew[1])<Sup(y_ein[1])) 
     cout << " erste Komp. obere Schranke:  Ja" << endl;
   else
     cout << " erste Komp. obere Schranke:  Nein" << endl;


  
   if (Inf(ynew[2]) > Inf(y_ein[2])) 
     cout << " zweite Komp. untere Schranke:  Ja" << endl;
    else
     cout << " zweite Komp. untere Schranke:  Nein" << endl;


   if (Sup(ynew[2])<Sup(y_ein[2]))
     cout << " zweite Komp. obere Schranke:  Ja" << endl;
    else
     cout << " zweite Komp. obere Schranke:  Nein" << endl;


   cout << Scientific << SetPrecision(8,6) << endl;

   cout << "yein[1]: " << y_ein[1] << endl;
   cout << "yein[2]: " << y_ein[2] << endl << endl;  

   cout << "yneu[1]: " << ynew[1] << endl; 
   cout << "yneu[2]: " << ynew[2] << endl << endl;
}

//-------------------------------------------------------------------------

int main()
{
   init_alpha();

   ivector x_ein(2); //beinhaltet geschaetzte Intervalle, wo NS liegen koennte

   int anz=5;
   imatrix x_matrix(anz,2);
   
   real epsi=0.0000000000;
   interval blow_up=_interval(-epsi, epsi);

//--------------------------------------------------------------------------
   //Nullstelle S.10
   x_matrix[1][1]=_interval(pred(0.548749998), succ(0.548750009)); //X1
   x_matrix[1][2]=_interval(pred(0.23438417), succ(0.23438419));


   //Nullstellen S.11
   x_matrix[2][1]=_interval(pred(0.50212045005), succ(0.50212045007)); //X2
   x_matrix[2][2]=_interval(pred(0.23546280357), succ(0.23546280359));

   x_matrix[3][1]=_interval(pred(0.514782868610), succ(0.514782868615))+blow_up; //X3
   x_matrix[3][2]=_interval(pred(0.235443268066), succ(0.235443268069))+blow_up;

   x_matrix[4][1]=_interval(pred(0.53502769900), succ(0.53502769902)); //X4
   x_matrix[4][2]=_interval(pred(0.23503240116), succ(0.23503240118));

   //Nullstelle S.12
   x_matrix[5][1]=_interval(pred(0.549779186509), succ(0.549779186515)); //X5
   x_matrix[5][2]=_interval(pred(0.249772912617), succ(0.249772912623));

//-------------------------------------------------------------------------
 
   for(int i=1; i<=anz; i++)
     {
       cout << i << endl;
       x_ein[1]=  x_matrix[i][1];
       x_ein[2]=  x_matrix[i][2]; 

       verif_schritt(x_ein); 
       cout << "--------------------------------------------------------" << endl;      
     }
//------------------------------------------------------------------------


}


/*
--------------------------------------------------------------------------

Testergebnisse  fuer verschiedene m (siehe Parameter oben)

--------------------------------------------------------------------------

m=2
1
Argumentintervalle: 
Xein[1]: [  0.548749,  0.548751]
Xein[2]: [  0.234384,  0.234385]

 erste Komp. untere Schranke:  Nein
 erste Komp. obere Schranke:  Nein
 zweite Komp. untere Schranke:  Nein
 zweite Komp. obere Schranke:  Nein

yein[1]: [-5.500001E-009,5.500001E-009]
yein[2]: [-1.000001E-008,1.000001E-008]

yneu[1]: [-1.797003E-004,1.850084E-004]
yneu[2]: [-1.930529E-005,1.943127E-005]

--------------------------------------------------------
2
Argumentintervalle: 
Xein[1]: [5.021204E-001,5.021205E-001]
Xein[2]: [2.354628E-001,2.354629E-001]

 erste Komp. untere Schranke:  Nein
 erste Komp. obere Schranke:  Nein
 zweite Komp. untere Schranke:  Nein
 zweite Komp. obere Schranke:  Nein

yein[1]: [-1.000012E-011,1.000012E-011]
yein[2]: [-1.000003E-011,1.000003E-011]

yneu[1]: [-3.300704E-005,3.627344E-005]
yneu[2]: [-1.391951E-005,1.218944E-005]

--------------------------------------------------------
3
Argumentintervalle: 
Xein[1]: [5.147828E-001,5.147829E-001]
Xein[2]: [2.354432E-001,2.354433E-001]

 erste Komp. untere Schranke:  Nein
 erste Komp. obere Schranke:  Nein
 zweite Komp. untere Schranke:  Nein
 zweite Komp. obere Schranke:  Nein

yein[1]: [-2.500112E-012,2.500112E-012]
yein[2]: [-1.500023E-012,1.500051E-012]

yneu[1]: [-2.168140E-005,2.024335E-005]
yneu[2]: [-2.204183E-005,1.181883E-005]

--------------------------------------------------------
4
Argumentintervalle: 
Xein[1]: [5.350276E-001,5.350277E-001]
Xein[2]: [2.350324E-001,2.350325E-001]

 erste Komp. untere Schranke:  Nein
 erste Komp. obere Schranke:  Nein
 zweite Komp. untere Schranke:  Nein
 zweite Komp. obere Schranke:  Nein

yein[1]: [-1.000012E-011,1.000012E-011]
yein[2]: [-1.000003E-011,1.000003E-011]

yneu[1]: [-1.785060E-005,1.706792E-005]
yneu[2]: [-1.483952E-005,1.428322E-005]

--------------------------------------------------------
5
Argumentintervalle: 
Xein[1]: [5.497791E-001,5.497792E-001]
Xein[2]: [2.497729E-001,2.497730E-001]

 erste Komp. untere Schranke:  Nein
 erste Komp. obere Schranke:  Nein
 zweite Komp. untere Schranke:  Nein
 zweite Komp. obere Schranke:  Nein

yein[1]: [-3.000156E-012,3.000045E-012]
yein[2]: [-3.000045E-012,3.000017E-012]

yneu[1]: [-1.458588E-006,5.653497E-006]
yneu[2]: [-1.476927E-005,3.104631E-005]

--------------------------------------------------------
m=6 : Groessenordnungen stimmen wenigstens ueberein !
1
Argumentintervalle: 
Xein[1]: [  0.548749,  0.548751]
Xein[2]: [  0.234384,  0.234385]

 erste Komp. untere Schranke:  Nein
 erste Komp. obere Schranke:  Nein
 zweite Komp. untere Schranke:  Nein
 zweite Komp. obere Schranke:  Nein

yein[1]: [-5.500001E-009,5.500001E-009]
yein[2]: [-1.000001E-008,1.000001E-008]

yneu[1]: [-5.771511E-007,5.734997E-007]
yneu[2]: [-6.042639E-007,6.047535E-007]

--------------------------------------------------------
2
Argumentintervalle: 
Xein[1]: [5.021204E-001,5.021205E-001]
Xein[2]: [2.354628E-001,2.354629E-001]

 erste Komp. untere Schranke:  Nein
 erste Komp. obere Schranke:  Nein
 zweite Komp. untere Schranke:  Nein
 zweite Komp. obere Schranke:  Nein

yein[1]: [-1.000012E-011,1.000012E-011]
yein[2]: [-1.000003E-011,1.000003E-011]

yneu[1]: [-1.858792E-011,1.900936E-011]
yneu[2]: [-3.007051E-011,2.405468E-011]

--------------------------------------------------------
3
Argumentintervalle: 
Xein[1]: [5.147828E-001,5.147829E-001]
Xein[2]: [2.354432E-001,2.354433E-001]

 erste Komp. untere Schranke:  Ja
 erste Komp. obere Schranke:  Ja
 zweite Komp. untere Schranke:  Nein
 zweite Komp. obere Schranke:  Nein

yein[1]: [-2.500112E-012,2.500112E-012]
yein[2]: [-1.500023E-012,1.500051E-012]

yneu[1]: [-1.668138E-012,1.982581E-012]
yneu[2]: [-1.509517E-012,1.532737E-012]

--------------------------------------------------------
4
Argumentintervalle: 
Xein[1]: [5.350276E-001,5.350277E-001]
Xein[2]: [2.350324E-001,2.350325E-001]

 erste Komp. untere Schranke:  Nein
 erste Komp. obere Schranke:  Nein
 zweite Komp. untere Schranke:  Nein
 zweite Komp. obere Schranke:  Nein

yein[1]: [-1.000012E-011,1.000012E-011]
yein[2]: [-1.000003E-011,1.000003E-011]

yneu[1]: [-1.569289E-011,1.634213E-011]
yneu[2]: [-1.607722E-011,1.711316E-011]

--------------------------------------------------------
5
Argumentintervalle: 
Xein[1]: [5.497791E-001,5.497792E-001]
Xein[2]: [2.497729E-001,2.497730E-001]

 erste Komp. untere Schranke:  Nein
 erste Komp. obere Schranke:  Nein
 zweite Komp. untere Schranke:  Nein
 zweite Komp. obere Schranke:  Ja

yein[1]: [-3.000156E-012,3.000045E-012]
yein[2]: [-3.000045E-012,3.000017E-012]

yneu[1]: [-3.003406E-012,3.181738E-012]
yneu[2]: [-3.116019E-012,2.696676E-012]

--------------------------------------------------------
m=8
1
Argumentintervalle: 
Xein[1]: [  0.548749,  0.548751]
Xein[2]: [  0.234384,  0.234385]

 erste Komp. untere Schranke:  Nein
 erste Komp. obere Schranke:  Nein
 zweite Komp. untere Schranke:  Nein
 zweite Komp. obere Schranke:  Nein

yein[1]: [-5.500001E-009,5.500001E-009]
yein[2]: [-1.000001E-008,1.000001E-008]

yneu[1]: [-2.126241E-005,2.125879E-005]
yneu[2]: [-2.233872E-005,2.233924E-005]

--------------------------------------------------------
2
Argumentintervalle: 
Xein[1]: [5.021204E-001,5.021205E-001]
Xein[2]: [2.354628E-001,2.354629E-001]

 erste Komp. untere Schranke:  Nein
 erste Komp. obere Schranke:  Nein
 zweite Komp. untere Schranke:  Nein
 zweite Komp. obere Schranke:  Nein

yein[1]: [-1.000012E-011,1.000012E-011]
yein[2]: [-1.000003E-011,1.000003E-011]

yneu[1]: [-1.221060E-009,1.221877E-009]
yneu[2]: [-1.760383E-009,1.756312E-009]

--------------------------------------------------------
3
Argumentintervalle: 
Xein[1]: [5.147828E-001,5.147829E-001]
Xein[2]: [2.354432E-001,2.354433E-001]

 erste Komp. untere Schranke:  Nein
 erste Komp. obere Schranke:  Nein
 zweite Komp. untere Schranke:  Nein
 zweite Komp. obere Schranke:  Nein

yein[1]: [-2.500112E-012,2.500112E-012]
yein[2]: [-1.500023E-012,1.500051E-012]

yneu[1]: [-1.833713E-010,1.842100E-010]
yneu[2]: [-1.537086E-010,1.525942E-010]

--------------------------------------------------------
4
Argumentintervalle: 
Xein[1]: [5.350276E-001,5.350277E-001]
Xein[2]: [2.350324E-001,2.350325E-001]

 erste Komp. untere Schranke:  Nein
 erste Komp. obere Schranke:  Nein
 zweite Komp. untere Schranke:  Nein
 zweite Komp. obere Schranke:  Nein

yein[1]: [-1.000012E-011,1.000012E-011]
yein[2]: [-1.000003E-011,1.000003E-011]

yneu[1]: [-1.249673E-009,1.252179E-009]
yneu[2]: [-1.296284E-009,1.295802E-009]

--------------------------------------------------------
5
Argumentintervalle: 
Xein[1]: [5.497791E-001,5.497792E-001]
Xein[2]: [2.497729E-001,2.497730E-001]

 erste Komp. untere Schranke:  Nein
 erste Komp. obere Schranke:  Nein
 zweite Komp. untere Schranke:  Nein
 zweite Komp. obere Schranke:  Nein

yein[1]: [-3.000156E-012,3.000045E-012]
yein[2]: [-3.000045E-012,3.000017E-012]

yneu[1]: [-2.944332E-010,2.935739E-010]
yneu[2]: [-2.767603E-010,2.758371E-010]

-------------------------------------------------------

m=10

1
Argumentintervalle: 
Xein[1]: [  0.548749,  0.548751]
Xein[2]: [  0.234384,  0.234385]

 erste Komp. untere Schranke:  Nein
 erste Komp. obere Schranke:  Nein
 zweite Komp. untere Schranke:  Nein
 zweite Komp. obere Schranke:  Nein

yein[1]: [-5.500001E-009,5.500001E-009]
yein[2]: [-1.000001E-008,1.000001E-008]

yneu[1]: [-7.659337E-004,7.659301E-004]
yneu[2]: [-8.047813E-004,8.047818E-004]
--------------------------------------------------------
2
Argumentintervalle: 
Xein[1]: [5.021204E-001,5.021205E-001]
Xein[2]: [2.354628E-001,2.354629E-001]

 erste Komp. untere Schranke:  Nein
 erste Komp. obere Schranke:  Nein
 zweite Komp. untere Schranke:  Nein
 zweite Komp. obere Schranke:  Nein

yein[1]: [-1.000012E-011,1.000012E-011]
yein[2]: [-1.000003E-011,1.000003E-011]

yneu[1]: [-4.459435E-008,4.459517E-008]
yneu[2]: [-6.419773E-008,6.419366E-008]

--------------------------------------------------------
3
Argumentintervalle: 
Xein[1]: [5.147828E-001,5.147829E-001]
Xein[2]: [2.354432E-001,2.354433E-001]

 erste Komp. untere Schranke:  Nein
 erste Komp. obere Schranke:  Nein
 zweite Komp. untere Schranke:  Nein
 zweite Komp. obere Schranke:  Nein

yein[1]: [-2.500112E-012,2.500112E-012]
yein[2]: [-1.500023E-012,1.500051E-012]

yneu[1]: [-6.792733E-009,6.793571E-009]
yneu[2]: [-5.661240E-009,5.660126E-009]

--------------------------------------------------------
4
Argumentintervalle: 
Xein[1]: [5.350276E-001,5.350277E-001]
Xein[2]: [2.350324E-001,2.350325E-001]

 erste Komp. untere Schranke:  Nein
 erste Komp. obere Schranke:  Nein
 zweite Komp. untere Schranke:  Nein
 zweite Komp. obere Schranke:  Nein

yein[1]: [-1.000012E-011,1.000012E-011]
yein[2]: [-1.000003E-011,1.000003E-011]

yneu[1]: [-4.578529E-008,4.578779E-008]
yneu[2]: [-4.743814E-008,4.743766E-008]

--------------------------------------------------------
5
Argumentintervalle: 
Xein[1]: [5.497791E-001,5.497792E-001]
Xein[2]: [2.497729E-001,2.497730E-001]

 erste Komp. untere Schranke:  Nein
 erste Komp. obere Schranke:  Nein
 zweite Komp. untere Schranke:  Nein
 zweite Komp. obere Schranke:  Nein

yein[1]: [-3.000156E-012,3.000045E-012]
yein[2]: [-3.000045E-012,3.000017E-012]

yneu[1]: [-1.082954E-008,1.082868E-008]
yneu[2]: [-1.017745E-008,1.017652E-008]

--------------------------------------------------------

 */
