/*
**  SLOPE_FOR: C-XSC classes for one- and multidimensional (interval)
**             slope arithmetic in forward mode
**
**  Copyright (C) 1999-2001 M. Braeuer, W. Hofschuster, W. Kraemer
**                          Wiss. Rechnen/Softwaretechnologie
**                          Universitaet Wuppertal, Germany   
**
**  This library is free software; you can redistribute it and/or
**  modify it under the terms of the GNU Library General Public
**  License as published by the Free Software Foundation; either
**  version 2 of the License, or (at your option) any later version.
**
**  This library is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
**  Library General Public License for more details.
**
**  You should have received a copy of the GNU Library General Public
**  License along with this library; if not, write to the Free
**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "slope_ari.h"

using namespace std;
using namespace cxsc; 

int main(){

  //Beispielprogramm fuer Steigungsarithmetik

  //eindim. Steigungsarithmetik

  interval arg=_interval(0.75,1.75);
  slope_type s=init_mid_var(arg);

  slope_type s1;

  s1=(sqrt(s));
  cout << "Nummer 0: "  << endl; print_slope_type(s1);cout << endl;

  s1=sqrt(s,2);
  cout << "Nummer 0: "  << endl; print_slope_type(s1);cout << endl;

  s1=(s+sin(s))*exp(-sqr(s));
  cout << "Nummer 1: "  << endl; print_slope_type(s1);cout << endl;

  slope_type s2;
  s2=power(s,4)-_real(10.0)*power(s,3)+_real(35.0)*sqr(s);
  s2=s2-_real(50.0)*s+_real(24.0);
  cout << "Nummer 2: "  << endl; print_slope_type(s2);cout << endl;

  slope_type s3;
  s3=sqr(ln(s+_real(1.25))-_real(0.84)*s);
  cout << "Nummer 3: "  << endl; print_slope_type(s3);cout << endl;

  slope_type s4;
  s4=_interval(2.0)/_interval(100.0)*sqr(s);
  s4=s4-_interval(3.0)/_interval(100.0)*exp(-sqr(_real(20.0)*(s-_real(0.875))));
  cout << "Nummer 4: "  << endl; print_slope_type(s4);cout << endl;

  slope_type s5;
  s5=exp(sqr(s));
  cout << "Nummer 5: "  << endl; print_slope_type(s5);cout << endl;






  cout << "Bsp. nichtdiff. Funktionen: " << endl;
  interval x2=_interval(1,7);
  slope_type s10=init_mid_var(x2);

  s10=chi(s10-_real(2),
	sqr(s10)-_real(20)*s10+_real(34),
        max(_real(2)*s10-_real(8), abs(s10-_real(2))-_real(2) )   );

  print_slope_type(s10);





  //mehrdimensionale Steigungsarithmetik

  ivector x_ein(2); //Intervallvektor mit Werten der Abhaengigen

  x_ein[1]=_interval(1,7); //Initialisiere x_ein
  x_ein[2]=_interval(0,0);

  slope_vector x;

  x=Init_mid_Var(x_ein); //Initialisiere Steigungswektor der Abhaengigen
                         //Entwicklungspunkte sind  Mittelpunkte

  slope erg=chi(x[1]-_real(2),
		sqr(x[1])-_real(20)*x[1]+_real(34),
		max(_real(2)*x[1]-_real(8), abs(x[1]-_real(2))-_real(2) ) );
  erg=erg+x[2];

  print_slope(erg);


  cout << endl << endl;

  x_ein[1]=_interval(8,8.25); //Initialisiere x_ein neu
  x_ein[2]=_interval(3,3.25);  

  ivector c_ein(2);
  c_ein[1]=_interval(8.25); //Initialisiere c_ein
  c_ein[2]=_interval(3.25);

  x=InitVar(x_ein,c_ein);

  print_slope_vector(x);  

  erg=_real(4.0)*sqr(x[1])-_interval(21)/_interval(10)*power(x[1],4)+
      _interval(1.0)/_interval(3.0)*power(x[1],6)+
      x[1]*x[2]-_real(4.0)*sqr(x[2])+_real(4.0)*power(x[2],4);

  print_slope(erg); 
 
} 


/* Ausgabe:
Nummer 0: 
f_x [0.86602540378443,1.32287565553230]
f_c [1.11803398874989,1.11803398874990]
f_s [0.40968333356480,0.50401716993092]


Nummer 0: 
f_x [0.86602540378443,1.32287565553230]
f_c [1.11803398874989,1.11803398874990]
f_s [0.40968333356480,0.50401716993092]


Nummer 1: 
f_x [0.06695863583529,1.56690276801004]
f_c [0.46093221638705,0.46093221638706]
f_s [-2.33760041128394,0.05214283851546]


Nummer 2: 
f_x [-9.70898437500001E+001,9.88476562500001E+001]
f_c [-0.90234375000001,-0.90234374999999]
f_s [-4.38750000000001E+001,3.82500000000001E+001]


Nummer 3: 
f_x [0.00000000000000,0.60350030307197]
f_c [0.01787816838274,0.01787816838275]
f_s [-0.15919845709498,0.43284195883022]


Nummer 4: 
f_x [-0.01875000000001,0.06125000000001]
f_c [0.03124999999999,0.03125000000001]
f_s [0.03999999999999,0.32666666666667]


Nummer 5: 
f_x [1.75505465696029,2.13809427591234E+001]
f_c [4.77073318196760,4.77073318196761]
f_s [6.03135705001460,3.32204191543115E+001]


Bsp. nichtdiff. Funktionen (vgl. Vorlesung): 
f_x [-1.05000000000000E+002,6.30000000000000E+001]
f_c [-0.00000000000000,0.00000000000000]
f_s [-15.00000000000000,2.00000000000000]

Slope Objekt mit Dimension 2
f_x   : [-1.05000000000000E+002,6.30000000000000E+001]
f_c   : [-0.00000000000000,0.00000000000000]
f_s[1]: [-15.00000000000000,2.00000000000000]
f_s[2]: [1.00000000000000,1.00000000000000]

Slope Objekt mit Dimension 2
f_x   : [7.82148251302082E+004,9.72076604980470E+004]
f_c   : [9.60747522949218E+004,9.60747522949220E+004]
f_s[1]: [6.62256649088539E+004,7.19989316406251E+004]
f_s[2]: [4.71312499999999E+002,5.32500000000001E+002]
*/


